@@ -162,6 +162,7 @@ class RandomRolloutSimulation(simulation.Simulation):
         self.time_of_intervention_list = []
         self.next_up_intervention_entry = 0
         self.use_uniform_rate = True
+        self.vax_queue = []
 
     def simtype(self):
         print('I am a simulation class of type Multi intervention Random Intervention')
@@ -172,6 +173,8 @@ class RandomRolloutSimulation(simulation.Simulation):
         self.proportion_reduced_list = proportion_reduced_list
         for i in range(len(intervention_gen_list)):
             self.intervened_status_list.append(False)
+        self.vax_queue = np.arange(self._N)
+        np.random.shuffle(self.vax_queue)
 
     def run_sim(self, with_memberships=False, uniform_rate=True, wait_for_recovery=False, visualize=False,
                 viz_graph=None, viz_pos=None, p_zero=None, kill_by=None, record_active_gen_sizes=False):
@@ -207,37 +210,29 @@ class RandomRolloutSimulation(simulation.Simulation):
     def intervene(self, intervention_entry):
         print('intervening')
         if self._N == 0:
-            self._N = len(self._A[0])
+            self._N = len(self._adjlist)
         frac_of_network = self.proportion_reduced_list[intervention_entry] * self._N
         how_many = 0
         if frac_of_network > 1:
             how_many = int(np.round(frac_of_network, 0))
         vaccinated_nodes = []
-        vax_labels = []
-        while len(vaccinated_nodes) < how_many:
-            random_set = np.unique(np.random.randint(0, self._N, how_many)) #TODO need to fix random set here like above
-            for node_label in random_set:
-                if len(vax_labels) < how_many:
-                    if node_label not in vax_labels:
-                        if self.use_uniform_rate:
-                            candidate_node = network.Node(node_label, -1, None, self._uniform_gamma)
-                        else:
-                            candidate_node = network.Node(node_label, -1, None, self._Gamma[node_label])
-                        if self.track_memberships:
-                            candidate_node.set_membership(self._node_memberships[candidate_node.get_label()])
-                        existing_node = self._existing_node(candidate_node)
-                        # all the nodes become active
-                        if existing_node.get_state() != nodestate.NodeState.VACCINATED:
-                            # these should also be added to the active nodes
-                            if existing_node.get_state() != nodestate.NodeState.INFECTED:
-                                existing_node.vaccinate()
-                                vaccinated_nodes.append(existing_node)
-                                vax_labels.append(node_label)
-                            # self._Beta[node_label] = np.full(self._N, self.beta_redux_list[intervention_entry])
-                            # self._Beta[:, node_label] = np.full(self._N, self.beta_redux_list[intervention_entry]).T
-                                self._update_IS_events(recovery_event=existing_node)
-                            # else:
-                            #     print("I was already infected")
+        random_set = self.vax_queue[:how_many]
+        for i, node_label in enumerate(random_set):
+            if self.use_uniform_rate:
+                candidate_node = network.Node(node_label, -1, None, self._uniform_gamma)
+            else:
+                candidate_node = network.Node(node_label, -1, None, self._Gamma[node_label])
+            if self.track_memberships:
+                candidate_node.set_membership(self._node_memberships[candidate_node.get_label()])
+            existing_node = self._existing_node(candidate_node)
+            # all the nodes become active
+            if existing_node.get_state() != nodestate.NodeState.VACCINATED:
+                # these should also be added to the active nodes
+                if existing_node.get_state() != nodestate.NodeState.INFECTED:
+                    existing_node.vaccinate()
+                    vaccinated_nodes.append(existing_node)
+                    self._update_IS_events(recovery_event=existing_node)
+        self.vax_queue = self.vax_queue[how_many:]
         return vaccinated_nodes
 
 
@@ -326,13 +321,14 @@ class TargetedInterventionSim(simulation.Simulation):
                     if existing_node.get_state() != nodestate.NodeState.VACCINATED:
                         if existing_node.get_state() != nodestate.NodeState.INFECTED: #Do not vaccinate currently infected nodes
                             existing_node.vaccinate()
-                            num_vaccinated += 1 #Only counted if high degree nodes are successfully vaccinated
+                            num_vaccinated += 1
                         else:
-                            infct_hd_nodes += 1 #Counts high-degree currently infected nodes
-                            print(degree)
+                            infct_hd_nodes += 1  # Counts high-degree currently infected nodes
+                    else:
+                        num_vaccinated += 1  # INCREMENT ANYWAY, still counts as one of the vaccinated quota despite not being elligible.
                     self._update_IS_events(recovery_event=existing_node)
         print(f"Number of already infected nodes: {infct_hd_nodes}, which is {(infct_hd_nodes/how_many)} percent of the projected vacine proportion")
-        print(f"In total, {num_vaccinated} were vaccinated, out of a planned {how_many}, with max degree {degree}")
+
 
 class TargetedRolloutSimulation(simulation.Simulation):
     def __init__(self, N, adjlist=None, max_unitless_sim_time=1000000, membership_groups=None, node_memberships=None):
@@ -410,7 +406,7 @@ class TargetedRolloutSimulation(simulation.Simulation):
         num_vaccinated = 0
         infct_hd_nodes = 0
         for degree in keys_decreasing:
-            current_node_group = degree_classes[degree] # To match analytical prediction, once a degree class is bumped up to meet the quota, the whole class is vaccinated
+            current_node_group = degree_classes[degree]
             if num_vaccinated < how_many:
                 for node in current_node_group:
                     if self.use_uniform_rate:
@@ -423,9 +419,11 @@ class TargetedRolloutSimulation(simulation.Simulation):
                     if existing_node.get_state() != nodestate.NodeState.VACCINATED:
                         if existing_node.get_state() != nodestate.NodeState.INFECTED: #Do not vaccinate currently infected nodes
                             existing_node.vaccinate()
-                            num_vaccinated += 1 #Only counted if high degree nodes are successfully vaccinated
+                            num_vaccinated += 1
                         else:
-                            infct_hd_nodes += 1 #Counts high-degree currently infected nodes
+                            infct_hd_nodes += 1  # Counts high-degree currently infected nodes
+                    else:
+                        num_vaccinated += 1 # INCREMENT ANYWAY, still counts as one of the vaccinated quota despite not being elligible.
                     self._update_IS_events(recovery_event=existing_node)
 
 class RingInterventionSim(simulation.Simulation):
